using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;

namespace Lotosummary.Tabulate
{
    class RandomNumber
    {
        const int B00000001 = 1;
        const int B00000010 = 2;
        const int B00000100 = 4;
        const int B00001000 = 8;
        const int B00010000 = 16;
        const int B00100000 = 32;
        const int BOOL_STREAM_SIZE = 192; // 32 * GROUP_SIZE
        const int GROUP_SIZE = 6;
        const int KB = 1024;
        const int KILOBYTES = 32;
        const int MASK = B00000001;
        const string COLON = ":";
        const string GAP = " ";
        const string RANNU_CSV = "rannu.csv";
        const string RANNU_TXT = "rannu.txt";
        const string SEPARATOR = "\\";

        bool csv, grouping, produce_rannu_text;
        string rannuText, separator;
        int from, interval_0, interval_1, kilo, prng, seed;
        Rei.Random.RandomBase prn;
        string[] b = { "00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26", "27", "28", "29", "30", "31", "32", "33", "34", "35", "36", "37", "38", "39", "40", "41", "42", "43", "44", "45", "46", "47", "48", "49", "50", "51", "52", "53", "54", "55", "56", "57", "58", "59", "60", "61", "62", "63", "64", "65", "66", "67", "68", "69", "70", "71", "72", "73", "74", "75", "76", "77", "78", "79", "80", "81", "82", "83", "84", "85", "86", "87", "88", "89", "90", "91", "92", "93", "94", "95", "96", "97", "98", "99" };

        private string[] driveDirectoryBase(string s) {
            const string SLASH = "/";
            const string YEN = "\\";
            string[] UNDIVIDED = {"", "", "", ""};
            int firstTrace, trace, w;
            int s_length = s.Length;
            string c; 
            string SEPARATOR = YEN;
            string[] part = {"", "", "", ""};
            part[0] = s; // s.length > 0; in other words, part[0].length > UNDIVIDED[0].length

// s is divided into three parts: drive name, directory name, and base name.
            int k = 0;
            for( ; k < s_length; ++k) {
                c = s.Substring(k, 1);
                if(c == YEN) {
                    SEPARATOR = YEN;
                    break;
                } else if(c == SLASH) {
                    SEPARATOR = SLASH;
                    break;
                }

            }

            if(k == s_length) {
                return UNDIVIDED;
            } else { // SEPARATOR has been found.
                w = k + 1;
                part[1] = s.Substring(0, w); // C:\ for example
                trace = ++k;
                firstTrace = trace;
            }

            for( ; k < s_length; ++k) {
                c = s.Substring(k, 1);
                if(c == SEPARATOR) {
                    trace = ++k;
                }

            }

            w = trace - firstTrace;
            part[2] = s.Substring(firstTrace, w);
            part[3] = s.Substring(trace); // manage.txt for example
            return part;
        }

        public uint random_integer()
        {
            return prn.NextUInt32();
        }

        public void setPRNG()
        {
            const int MERSENNE_TWISTER = 1; // MT
            const int RANROT_B = 3;
            const int SFMT_PRNG = 4;
            const int WELL = 5;
            const int XORSHIFT = 6;

            switch (prng)
            {
                case MERSENNE_TWISTER: // MT
                    prn = new Rei.Random.MersenneTwister(seed);
                    break;
                case RANROT_B:
                    prn = new Rei.Random.RanrotB(seed);
                    break;
                case SFMT_PRNG:
                    prn = new Rei.Random.SFMT(seed);
                    break;
                case WELL:
                    prn = new Rei.Random.Well(seed);
                    break;
                case XORSHIFT:
                    prn = new Rei.Random.Xorshift(seed);
                    break;
                default:
                    break;
            }

            // idling
            for (int k = 0; k < from; ++k)
            {
                random_integer();
            }

        }

        public bool createRannuText(string pathName, int pr, bool ra)
        {
            prng = pr;
            produce_rannu_text = ra;

            // rannuText (RANdom-NUmber table TEXT)
            double difference, range;
            int bytes, k, plus, size;
            uint g;
            string item;

            string[] name = {"", "", "", ""};
            name = driveDirectoryBase(pathName);
            string directoryName = name[1] + name[2]; // C:\RannuCS\ for example

            if (csv)
            {
                separator = ",";
                rannuText = RANNU_CSV;
            }
            else
            {
                separator = " ";
                rannuText = RANNU_TXT;
            }

// Redirecting unless rannuText is opened.
            Feedable f;
            if(produce_rannu_text) {
                pathName = directoryName + rannuText;
                f = new FileOutputNewLine(pathName);
            } else {
                f = new StandardOutputNewLine();
            }

            f.setting(separator);

            if (seed == 0)
            {
                DateTime localDate = DateTime.Now;
                var culture = new CultureInfo("en-US");
                string s = localDate.ToString(culture);
                /*
                1/23/2000 4:05:06 AM
                         ppqr  r    
                */
                int p = s.IndexOf(GAP);
                int q = s.IndexOf(COLON);
                p += 1;
                string HOUR = s.Substring(p, q - p);
                int r = q + 1;
                string MINUTE = s.Substring(r, 2);
                r += 3;
                string SECOND = s.Substring(r, 2);
                int hour = Convert.ToInt32(HOUR);
                int minute = Convert.ToInt32(MINUTE);
                int second = Convert.ToInt32(SECOND);
                int add = minute * 60 + hour * 60 * 60;
                second += add;
                seed = second;
            }

            setPRNG();

            if (interval_1 == 63)
            {
                if (interval_0 == 0)
                {
                    grouping = true;
                }

            }

            if (grouping)
            {
                // Buffering and grouping so that numbers can range from 0 to 63.
                uint[] boolStream = new uint[BOOL_STREAM_SIZE];
                uint bits, u;
                uint[] shift = {
                    B00000001,
                    B00000010,
                    B00000100,
                    B00001000,
                    B00010000,
                    B00100000
                };

                k = 0;
                bytes = kilo * KB;
                size = 0;
                while (size <= bytes)
                {
                    // Buffering
                    u = random_integer();
                    for (int i = 0; i < 32; ++i)
                    {
                        boolStream[k++] = MASK & u;
                        u >>= 1;
                    }

                    if (k < BOOL_STREAM_SIZE)
                    {
                        continue;
                    }

                    // Grouping
                    for (k = 0; k < BOOL_STREAM_SIZE; )
                    {
                        bits = 0;
                        for (int i = 0; i < GROUP_SIZE; ++i)
                        {
                            if (boolStream[k++] == 1)
                            {
                                bits |= shift[i];
                            }

                        }

                        item = b[bits];
                        size = f.writing(item);
                    }

                    k = 0;
                }

            }
            else
            {
                // Selecting an item and writing
                bytes = kilo * KB;
                difference = (double)(interval_1 - interval_0 + 1);
                range = difference / 4294967296.0;
                size = 0;
                while (size <= bytes)
                {
                    g = random_integer(); // 0 to 4294967295
                    plus = (int)Math.Floor(range * (double)g);
                    k = interval_0 + plus;
                    item = b[k];
                    size = f.writing(item);
                }

            }

            f.ending(csv);
            return true;
        }

        public RandomNumber(int s)
        {
            csv = false;
            from = 0;
            grouping = false;
            interval_0 = 1;
            interval_1 = 37;
            kilo = KILOBYTES;
            seed = s;
        }

        public RandomNumber(int s, int f, bool g)
        {
            csv = false;
            from = f; // idling
            grouping = g;
            interval_0 = 1;
            interval_1 = 37;
            kilo = KILOBYTES;
            seed = s;
        }

        public RandomNumber(int s, int f, int i, int l, int k, bool c, bool g)
        {
            csv = c;
            from = f; // idling
            grouping = g;
            interval_0 = i;
            interval_1 = l;
            kilo = k;
            seed = s;
        }

    }

}
