using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace Lotosummary.Tabulate
{
    public class Rannu
    {
        const int ERROR_CODE = 1;
        const int FROM = 400;
        const int INTERVAL_FROM = 1;
        const int INTERVAL_TO = 37;
        const int KILOBYTES = 32;
        const int LARGE_NUMBER = 2147483647;
        const int MANAGE_TEXT_LINE_COUNT = 9;
        const int MERSENNE_TWISTER = 1; // MT
        const int SEED = 123456;
        const string ERROR_TEXT = "error.txt";
        const string MANAGE_TEXT = "manage.txt";
        const string RANNU_CSV = "rannu.csv";
        const string RANNU_TEXT = "rannu.txt";
        const string SEPARATOR = "\\";

        bool csv, grouping, produce_rannu_text;
        int from = FROM; // idling
        int interval_0 = INTERVAL_FROM;
        int interval_1 = INTERVAL_TO;
        int kilo = KILOBYTES;
        int lineCount = LARGE_NUMBER;
        int prng = MERSENNE_TWISTER; // MT
        int seed = SEED;
        string error = "No error";
        string FULL_PATH_NAME, pathName;

        string[] m = {
            "",           // m[0], FULL_PATH_NAME includes MANAGE_TEXT.
            "0",          // m[1], csv: RANNU_CSV
            "C:\\",       // m[2], pathName includes the directory name (C:\) and RANNU_TEXT.
            "32",         // m[3], kilo: RANNU_TEXT size (kilobytes, KB)
            "1",          // m[4], interval_0: Loto-7
            "37",         // m[5], interval_1: Loto-7
            "0",          // m[6], grouping: interval of 0-63
            "400",        // m[7], from: idling
            "123456",     // m[8], seed: pseudorandom number generator (PRNG)
            "2147483647", // m[9], lineCount: the first part of RANNU_TEXT
        };

        private bool readManageText()
        {
            m[0] = FULL_PATH_NAME;

            var fs = new FileStream(FULL_PATH_NAME, FileMode.Open);
            try
            {
                var file = new StreamReader(fs, Encoding.GetEncoding(65001)); // UTF-8
                for (int line = 1; line <= MANAGE_TEXT_LINE_COUNT; line += 1)
                {
                    m[line] = file.ReadLine();
                    if (m[line] == null)
                    {
                        break;
                    }

                }

                file.Close();
                stringArrayToValues();
                return true;
            }
            catch (FileNotFoundException e3)
            {
                error = "FileNotFoundException e3" + e3.Message + " / " + FULL_PATH_NAME;
                createErrorText(error);
                return false;
            }
            catch (System.IO.IOException e4)
            {
                error = "System.IO.IOException e4" + e4.Message + " / " + FULL_PATH_NAME;
                createErrorText(error);
                return false;
            }
            catch (Exception e5)
            {
                error = "Exception e5" + e5.Message + " / " + FULL_PATH_NAME;
                createErrorText(error);
                return false;
            }

        }

        private string[] driveDirectoryBase(string s) 
        {
            const string SLASH = "/";
            const string YEN = "\\";
            string[] UNDIVIDED = {"", "", "", ""};
            int firstTrace, trace, w;
            int s_length = s.Length;
            string c;
            string SEPARATOR = YEN;
            string[] part = {"", "", "", ""};
            part[0] = s; // s.length > 0; in other words, part[0].length > UNDIVIDED[0].length

// s is divided into three parts: drive name, directory name, and base name.
            int k = 0;
            for( ; k < s_length; ++k) 
            {
                c = s.Substring(k, 1);
                if(c == YEN) 
                {
                    SEPARATOR = YEN;
                    break;
                } 
                else if(c == SLASH) 
                {
                    SEPARATOR = SLASH;
                    break;
                }

            }

            if(k == s_length) 
            {
                return UNDIVIDED;
            } 
            else 
            { // SEPARATOR has been found.
                w = k + 1;
                part[1] = s.Substring(0, w); // C:\ for example
                trace = ++k;
                firstTrace = trace;
            }

            for( ; k < s_length; ++k) 
            {
                c = s.Substring(k, 1);
                if(c == SEPARATOR) 
                {
                    trace = ++k;
                }

            }

            w = trace - firstTrace;
            part[2] = s.Substring(firstTrace, w);
            part[3] = s.Substring(trace); // manage.txt for example
            return part;
        }

        private void createErrorText(string messages)
        {
            var file = new FileStream(ERROR_TEXT, FileMode.Create);
            byte[] data = System.Text.Encoding.UTF8.GetBytes(messages);
            file.Write(data, 0, data.Length);
            file.Close();
        }

        private void stringArrayToValues()
        {
            if (m[1] == "0")
            {
                csv = false;
                pathName = m[2] + RANNU_TEXT;
            }
            else
            {
                csv = true;
                pathName = m[2] + RANNU_CSV;
            }

            kilo = Convert.ToInt32(m[3]);
            interval_0 = Convert.ToInt32(m[4]);
            interval_1 = Convert.ToInt32(m[5]);

            grouping = (m[6] == "0") ? false : true;
            from = Convert.ToInt32(m[7]); // idling
            seed = Convert.ToInt32(m[8]);
            lineCount = Convert.ToInt32(m[9]); // fileHeader() may read the first part of RANNU_TEXT.
        }

        public Rannu()
        {
            // C:\RannuCS\manage.txt for example
            string firstPart = System.IO.Directory.GetCurrentDirectory();
            string[] name = {"", "", "", ""};
            name = driveDirectoryBase(firstPart);
            string root = name[1];
            if (firstPart == root) // C:\ for example
            {
                FULL_PATH_NAME = root + MANAGE_TEXT;
            }
            else // C:\RannuCS for example 
            {
                FULL_PATH_NAME = firstPart + SEPARATOR + MANAGE_TEXT;
            }

        }

        public void run(int f, int i0, int i1, int k, int p, int s, bool c, bool g, bool ra)
        {
            readManageText(); // m[0] = FULL_PATH_NAME;

            from = f;
            interval_0 = i0;
            interval_1 = i1;
            kilo = k;
            prng = p;
            seed = s;
            csv = c;
            grouping = g;
            produce_rannu_text = ra;
            var r = new RandomNumber(seed, from, interval_0, interval_1, kilo, csv, grouping);
            if (r.createRannuText(pathName, prng, produce_rannu_text))
            {
                return;
            }
            else
            {
                error = "r.createRannuText(" 
                    + pathName 
                    + ", " 
                    + prng.ToString("D") 
                    + ", ";
                error += produce_rannu_text ? "true" : "false"; 
                error += ")";
                createErrorText(error);
                Environment.Exit(ERROR_CODE);
            }

        }

    }

}
